@@ -1,9 +1,18 @@
 # Graphs menu dialogs
 
-# last modified 11 June 03 by J. Fox
+# last modified 20 July 03 by J. Fox
 
 histogram <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) == 0){
+        tkmessageBox(message="There no numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Histogram")
     xFrame <- tkframe(top)
@@ -17,7 +26,7 @@ histogram <- function(){
         bins <- tclvalue(binsVariable)
         bins <- if (bins == "auto") "'Sturges'" else as.numeric(bins)
         scale <- tclvalue(scaleVariable) == "frequencies"
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         command <- paste("hist(", .activeDataSet, "$", x,
             ", breaks=", bins, sep="")
@@ -26,7 +35,7 @@ histogram <- function(){
         tkfocus(.commander)
         }
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }
@@ -34,7 +43,7 @@ histogram <- function(){
     OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
     cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(hist)
         }
     helpButton <- tkbutton(buttonsFrame, text="Help", width="12", command=onHelp)
@@ -64,14 +73,24 @@ histogram <- function(){
     tkwm.resizable(top, 0, 0)
     tkselection.set(xBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
 
 stemAndLeaf <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) == 0){
+        tkmessageBox(message="There no numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Stem and Leaf Display")
     xFrame <- tkframe(top)
@@ -80,18 +99,60 @@ stemAndLeaf <- function(){
     xScroll <- tkscrollbar(xFrame, repeatinterval=5, command=function(...) tkyview(xBox, ...))
     tkconfigure(xBox, yscrollcommand=function(...) tkset(xScroll, ...))
     for (x in .numeric) tkinsert(xBox, "end", x)
+    displayDigits <- tclVar("1")
+    onDigits <- function(...){
+        tclvalue(displayDigits) <- formatC(10^as.numeric(tclvalue(leafsDigitValue)), 
+            format="fg", big.mark=",")
+        tclvalue(leafsAutoVariable) <- "0"
+        }
+    optionsFrame <- tkframe(top)
+    partsFrame <- tkframe(top)
+    leafsFrame <- tkframe(top)
+    styleFrame <- tkframe(top)
+    leafsAutoVariable <- tclVar("1")
+    leafsDigitCheckBox <- tkcheckbutton(leafsFrame, variable=leafsAutoVariable)
+    partsVariable <- tclVar("auto")
+    partsAutoButton <- tkradiobutton(partsFrame, variable=partsVariable, value="auto")
+    parts1Button <- tkradiobutton(partsFrame, variable=partsVariable, value="1")
+    parts2Button <- tkradiobutton(partsFrame, variable=partsVariable, value="2")
+    parts5Button <- tkradiobutton(partsFrame, variable=partsVariable, value="5")
+    styleVariable <- tclVar("Tukey")
+    TukeyButton <- tkradiobutton(styleFrame, variable=styleVariable, value="Tukey")
+    bareButton <- tkradiobutton(styleFrame, variable=styleVariable, value="bare")
+    trimOutliersVariable <- tclVar("1")
+    trimOutliersCheckBox <- tkcheckbutton(optionsFrame, variable=trimOutliersVariable)    
+    showDepthsVariable <- tclVar("1")
+    showDepthsCheckBox <- tkcheckbutton(optionsFrame, variable=showDepthsVariable)
+    reverseNegativeVariable <- tclVar("1")
+    reverseNegativeCheckBox <- tkcheckbutton(optionsFrame, variable=reverseNegativeVariable)
+    leafsDigitValue <- tclVar("0")
+    leafsDigitSlider <- tkscale(leafsFrame, from=-6, to=6, showvalue=FALSE, variable=leafsDigitValue,
+        resolution=1, orient="horizontal", command=onDigits)
+    leafsDigitShow <- tklabel(leafsFrame, textvariable=displayDigits, width=8, justify="right")
     onOK <- function(){
         x <- .numeric[as.numeric(tkcurselection(xBox)) + 1]
-        stems <- as.character(tclvalue(stemsValue))
-        tkgrab.release(top)
+        unit <- if (tclvalue(leafsAutoVariable) == "1") "" 
+            else paste(", unit=", 10^as.numeric(tclvalue(leafsDigitValue)), sep="")
+        m <- if (tclvalue(partsVariable) == "auto") ""
+            else paste(", m=", tclvalue(partsVariable), sep="")
+        trim <- if (tclvalue(trimOutliersVariable) == "1") ""
+            else ", trim.outliers=FALSE"
+        depths <- if (tclvalue(showDepthsVariable) == "1") ""
+            else ", depths=FALSE"
+        reverse <- if (tclvalue(reverseNegativeVariable) == "1") ""
+            else ", reverse.negative.leaves=FALSE"
+        style <- if (tclvalue(styleVariable) == "Tukey") ""
+            else ', style="bare"'
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
-        command <- paste("stem(", .activeDataSet, "$", x, ", scale=", stems, ")", sep="")
+        command <- paste("stem.leaf(", .activeDataSet, "$", x, style, unit, m, trim, 
+            depths, reverse, ")", sep="")
         logger(command)
         justDoIt(command)
         tkfocus(.commander)
         }
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }    
@@ -99,38 +160,56 @@ stemAndLeaf <- function(){
     OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
     cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
-        help(stem)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
+        help(stem.leaf)
         }
-    helpButton <- tkbutton(top, text="Help", width="12", command=onHelp)
-    stemsFrame <- tkframe(top)
-    stemsValue <- tclVar("1")
-    stemSlider <- tkscale(stemsFrame, from=1, to=4, showvalue=TRUE, variable=stemsValue,
-        resolution=1, orient="horizontal")
-    tkgrid(tklabel(top, text="Variable (pick one)"), 
-        tklabel(top, text="Scale Factor"), sticky="w")
+    helpButton <- tkbutton(buttonsFrame, text="Help", width="12", command=onHelp)
+    tkgrid(tklabel(top, text="Variable (pick one)"), sticky="w")
     tkgrid(xBox, xScroll, sticky="nw")
-    tkgrid(xFrame, stemsFrame, sticky="nw")    
-    tkgrid(OKbutton, cancelButton, sticky="w")
-    tkgrid(buttonsFrame, helpButton, sticky="w")
+    tkgrid(xFrame, sticky="nw")
+    tkgrid(tklabel(leafsFrame, text="Leafs digit: Automatic"), leafsDigitCheckBox,
+        tklabel(leafsFrame, text="  or set:", fg="red"), leafsDigitShow, leafsDigitSlider, sticky="w")  
+    tkgrid(leafsFrame, sticky="w") 
+    tkgrid(tklabel(partsFrame, text="Parts per stem:   Automatic"), partsAutoButton,
+        tklabel(partsFrame, text="  1"), parts1Button, 
+        tklabel(partsFrame, text="  2"), parts2Button, 
+        tklabel(partsFrame, text="  5"), parts5Button, sticky="w")
+    tkgrid(partsFrame, sticky="w")
+    tkgrid(tklabel(styleFrame, text="Style of divided stems: Tukey"), TukeyButton,
+        tklabel(styleFrame, text="  repeated stem digits"), bareButton, sticky="w")
+    tkgrid(styleFrame, sticky="w")
+    tkgrid(tklabel(optionsFrame, text="Trim outliers"), trimOutliersCheckBox, sticky="w")
+    tkgrid(tklabel(optionsFrame, text="Show depths"), showDepthsCheckBox, sticky="w")
+    tkgrid(tklabel(optionsFrame, text="Reverse negative leaves"), reverseNegativeCheckBox, sticky="w")
+    tkgrid(optionsFrame, sticky="w")
+    tkgrid(OKbutton, cancelButton, tklabel(buttonsFrame, text="        "), helpButton, sticky="w")
+    tkgrid(buttonsFrame, sticky="w")
     tkgrid.configure(xScroll, sticky="ns")
-    tkgrid.configure(stemSlider, sticky="nw")
-    tkgrid.configure(helpButton, sticky="e")
-    for (row in 0:3) tkgrid.rowconfigure(top, row, weight=0)
-    for (col in 0:1) tkgrid.columnconfigure(top, col, weight=0)
+    tclvalue(leafsAutoVariable) <- "1"
+    for (row in 0:6) tkgrid.rowconfigure(top, row, weight=0)
+    for (col in 0:0) tkgrid.columnconfigure(top, col, weight=0)
     .Tcl("update idletasks")
     tkwm.resizable(top, 0, 0)
     tkselection.set(xBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
 
-
 boxPlot <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) == 0){
+        tkmessageBox(message="There no numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Boxplot")
     xFrame <- tkframe(top)
@@ -146,7 +225,7 @@ boxPlot <- function(){
     onOK <- function(){
         x <- as.character(tkget(xBox, "active"))
         identifyPoints <- "1" == tclvalue(identifyVariable)
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         var <- paste(.activeDataSet, "$", x, sep="")
         if (.groups == "FALSE") {
@@ -168,6 +247,15 @@ boxPlot <- function(){
         tkfocus(.commander)
         }
     onGroups <- function(){
+        if (length(.factors) == 0){
+            tkmessageBox(message="There no factors in the active data set.", 
+                    icon="error", type="ok")
+            tkwm.deiconify(top)
+            if (.grab.focus) tkgrab.set(top)
+            tkfocus(top)
+            tkwait.window(top)
+            return()
+            }
         subdialog <- tktoplevel()
         tkwm.title(subdialog, "Groups")
         groupsFrame <- tkframe(subdialog)
@@ -179,18 +267,18 @@ boxPlot <- function(){
         onOKsub <- function() {
             groups <- as.character(tkget(groupsBox, "active"))
             assign(".groups", groups, envir=.GlobalEnv)
-            tkgrab.release(subdialog)
+            if (.grab.focus) tkgrab.release(subdialog)
             tkdestroy(subdialog)
             tkwm.deiconify(top)
-            tkgrab.set(top)
+            if (.grab.focus) tkgrab.set(top)
             tkfocus(top)
             tkwait.window(top)
             }
         onCancelSub <- function() {
-            tkgrab.release(subdialog) 
+            if (.grab.focus) tkgrab.release(subdialog) 
             tkdestroy(subdialog) 
             tkwm.deiconify(top)
-            tkgrab.set(top)
+            if (.grab.focus) tkgrab.set(top)
             tkfocus(top)
             tkwait.window(top)
             }
@@ -208,14 +296,16 @@ boxPlot <- function(){
         .Tcl("update idletasks")
         tkwm.resizable(subdialog, 0, 0)
         tkbind(subdialog, "<Return>", onOKsub)
+        if (.double.click) tkbind(subdialog, "<Double-ButtonPress-1>", onOKsub)
+        tkbind(groupsBox, "<Double-ButtonPress-1>", onOKsub)
         tkselection.set(groupsBox, 0)
         tkwm.deiconify(subdialog)
-        tkgrab.set(subdialog)
+        if (.grab.focus) tkgrab.set(subdialog)
         tkfocus(subdialog)
         tkwait.window(subdialog)
         }
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }
@@ -223,7 +313,7 @@ boxPlot <- function(){
     OKbutton <- tkbutton(buttonFrame, text="OK", width="12", command=onOK, default="active")
     cancelButton <- tkbutton(buttonFrame, text="Cancel", width="12", command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(boxplot)
         }
     helpButton <- tkbutton(buttonFrame, text="Help", width="12", command=onHelp)
@@ -245,14 +335,24 @@ boxPlot <- function(){
     tkwm.resizable(top, 0, 0)
     tkselection.set(xBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
 
 scatterPlot <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) < 2){
+        tkmessageBox(message="There are fewer than 2 numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Scatterplot")
     xFrame <- tkframe(top)
@@ -282,12 +382,12 @@ scatterPlot <- function(){
     smoothCheckBox <- tkcheckbutton(optionsFrame, variable=smoothLine)
     sliderValue <- tclVar("50")
     slider <- tkscale(optionsFrame, from=0, to=100, showvalue=TRUE, variable=sliderValue,
-        resolution=1, orient="horizontal")
+        resolution=5, orient="horizontal")
     subsetVariable <- tclVar("<all valid cases>")
     subsetFrame <- tkframe(top)
     subsetEntry <- tkentry(subsetFrame, width="20", textvariable=subsetVariable)
     subsetScroll <- tkscrollbar(subsetFrame, orient="horizontal",
-        repeatinterval=5, command=function(...) tkyview(subsetEntry, ...))
+        repeatinterval=5, command=function(...) tkxview(subsetEntry, ...))
     tkconfigure(subsetEntry, xscrollcommand=function(...) tkset(subsetScroll, ...))
     assign(".groups", "FALSE", envir=.GlobalEnv)
     assign(".linesByGroup", "FALSE", envir=.GlobalEnv)
@@ -297,7 +397,7 @@ scatterPlot <- function(){
         if (x == y) {
             tkmessageBox(message="x and y variables must be different", 
                 icon="error", type="ok")
-            tkgrab.release(top)
+            if (.grab.focus) tkgrab.release(top)
             tkdestroy(top)
             scatterPlot()
             return()
@@ -311,9 +411,9 @@ scatterPlot <- function(){
         smooth <- as.character("1" == tclvalue(smoothLine))
         span <- as.numeric(tclvalue(sliderValue))
         subset <- tclvalue(subsetVariable)
-        subset <- if (subset == "<all valid cases>") "" 
+        subset <- if (trim.blanks(subset) == "<all valid cases>") "" 
             else paste(", subset=", subset, sep="")
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)
         if (.groups == "FALSE") {
@@ -331,6 +431,15 @@ scatterPlot <- function(){
             }
         }
     onGroups <- function(){
+        if (length(.factors) == 0){
+            tkmessageBox(message="There are no factors in the active data set.", 
+                    icon="error", type="ok")
+            tkwm.deiconify(top)
+            if (.grab.focus) tkgrab.set(top)
+            tkfocus(top)
+            tkwait.window(top)
+            return()
+            }
         subdialog <- tktoplevel()
         tkwm.title(subdialog, "Groups")
         linesByGroupFrame <- tkframe(subdialog)
@@ -348,18 +457,18 @@ scatterPlot <- function(){
             assign(".groups", groups, envir=.GlobalEnv)
             lines <- as.character("1" == tclvalue(linesByGroup))
             assign(".linesByGroup", lines, envir=.GlobalEnv)
-            tkgrab.release(subdialog)
+            if (.grab.focus) tkgrab.release(subdialog)
             tkdestroy(subdialog)
             tkwm.deiconify(top)
-            tkgrab.set(top)
+            if (.grab.focus) tkgrab.set(top)
             tkfocus(top)
             tkwait.window(top)
             }
         onCancelSub <- function() {
-            tkgrab.release(subdialog)  
+            if (.grab.focus) tkgrab.release(subdialog)  
             tkdestroy(subdialog)
             tkwm.deiconify(top)
-            tkgrab.set(top)
+            if (.grab.focus) tkgrab.set(top)
             tkfocus(top)
             tkwait.window(top)
             }
@@ -379,14 +488,15 @@ scatterPlot <- function(){
         .Tcl("update idletasks")
         tkwm.resizable(subdialog, 0, 0)
         tkbind(subdialog, "<Return>", onOKsub)
+        if (.double.click) tkbind(subdialog, "<Double-ButtonPress-1>", onOKsub)
         tkselection.set(groupsBox, 0)
         tkwm.deiconify(subdialog)
-        tkgrab.set(subdialog)
+        if (.grab.focus) tkgrab.set(subdialog)
         tkfocus(subdialog)
         tkwait.window(subdialog)
         }
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }
@@ -394,7 +504,7 @@ scatterPlot <- function(){
     OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
     cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(scatterplot)
         }
     helpButton <- tkbutton(top, text="Help", width="12", command=onHelp)
@@ -429,14 +539,24 @@ scatterPlot <- function(){
     tkselection.set(xBox, 0)
     tkselection.set(yBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
 
 scatterPlotMatrix <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) < 3){
+        tkmessageBox(message="There are fewer than 3 numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Scatterplot Matrix")
     variablesFrame <- tkframe(top)
@@ -453,7 +573,7 @@ scatterPlotMatrix <- function(){
     smoothCheckBox <- tkcheckbutton(optionsFrame, variable=smoothLine)
     sliderValue <- tclVar("50")
     slider <- tkscale(optionsFrame, from=0, to=100, showvalue=TRUE, variable=sliderValue,
-        resolution=1, orient="horizontal")
+        resolution=5, orient="horizontal")
     diagonalFrame <- tkframe(top)
     diagonal <- tclVar("density")
     histogramButton <- tkradiobutton(diagonalFrame, variable=diagonal, value="histogram")
@@ -465,7 +585,7 @@ scatterPlotMatrix <- function(){
     subsetFrame <- tkframe(top)
     subsetEntry <- tkentry(subsetFrame, width="20", textvariable=subsetVariable)
     subsetScroll <- tkscrollbar(subsetFrame, orient="horizontal",
-        repeatinterval=5, command=function(...) tkyview(subsetEntry, ...))
+        repeatinterval=5, command=function(...) tkxview(subsetEntry, ...))
     tkconfigure(subsetEntry, xscrollcommand=function(...) tkset(subsetScroll, ...))
     assign(".groups", "FALSE", envir=.GlobalEnv)
     assign(".linesByGroup", "FALSE", envir=.GlobalEnv)
@@ -483,9 +603,9 @@ scatterPlotMatrix <- function(){
         span <- as.numeric(tclvalue(sliderValue))
         diag <- as.character(tclvalue(diagonal))
         subset <- tclvalue(subsetVariable)
-        subset <- if (subset == "<all valid cases>") "" 
+        subset <- if (trim.blanks(subset) == "<all valid cases>") "" 
             else paste(", subset=", subset, sep="")
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         if (.groups == "FALSE") {
            command <- paste("scatterplot.matrix(~", paste(variables, collapse="+"),
@@ -507,6 +627,15 @@ scatterPlotMatrix <- function(){
         tkfocus(.commander)
         }
     onGroups <- function(){
+        if (length(.factors) == 0){
+            tkmessageBox(message="There are no factors in the active data set.", 
+                    icon="error", type="ok")
+            tkwm.deiconify(top)
+            if (.grab.focus) tkgrab.set(top)
+            tkfocus(top)
+            tkwait.window(top)
+            return()
+            }
         subdialog <- tktoplevel()
         tkwm.title(subdialog, "Groups")
         linesByGroupFrame <- tkframe(subdialog)
@@ -524,18 +653,18 @@ scatterPlotMatrix <- function(){
             assign(".groups", groups, envir=.GlobalEnv)
             lines <- as.character("1" == tclvalue(linesByGroup))
             assign(".linesByGroup", lines, envir=.GlobalEnv)
-            tkgrab.release(subdialog)
+            if (.grab.focus) tkgrab.release(subdialog)
             tkdestroy(subdialog)
             tkwm.deiconify(top)
-            tkgrab.set(top)
+            if (.grab.focus) tkgrab.set(top)
             tkfocus(top)
             tkwait.window(top)
             }
         onCancelSub <- function() {
-            tkgrab.release(subdialog)  
+            if (.grab.focus) tkgrab.release(subdialog)  
             tkdestroy(subdialog)
             tkwm.deiconify(top)
-            tkgrab.set(top)
+            if (.grab.focus) tkgrab.set(top)
             tkfocus(top)
             tkwait.window(top)
             }
@@ -555,13 +684,14 @@ scatterPlotMatrix <- function(){
         .Tcl("update idletasks")
         tkwm.resizable(subdialog, 0, 0)
         tkbind(subdialog, "<Return>", onOKsub)
+        if (.double.click) tkbind(subdialog, "<Double-ButtonPress-1>", onOKsub)
         tkwm.deiconify(subdialog)
-        tkgrab.set(subdialog)
+        if (.grab.focus) tkgrab.set(subdialog)
         tkfocus(subdialog)
         tkwait.window(subdialog)
         }
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }
@@ -569,7 +699,7 @@ scatterPlotMatrix <- function(){
     OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
     cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(scatterplot.matrix)
         }
     helpButton <- tkbutton(top, text="Help", width="12", command=onHelp)
@@ -602,14 +732,24 @@ scatterPlotMatrix <- function(){
     tkgrid.configure(variablesScroll, sticky="ns")
     tkgrid.configure(helpButton, sticky="e")
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
 
 barGraph <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.factors) == 0){
+        tkmessageBox(message="There are no factors in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Bar Graph")
     variableFrame <- tkframe(top)
@@ -621,7 +761,7 @@ barGraph <- function(){
     for (var in .factors) tkinsert(variableBox, "end", var)
     onOK <- function(){
         variable <- as.character(tkget(variableBox, "active"))
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         command <- paste("barplot(table(", variable, '), xlab="',
             variable, '", ylab="Frequency")', sep="")
@@ -632,13 +772,13 @@ barGraph <- function(){
     buttonsFrame <- tkframe(top)
     OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }
     cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(barplot)
         }
     helpButton <- tkbutton(buttonsFrame, text="Help", width="12", command=onHelp)
@@ -654,14 +794,25 @@ barGraph <- function(){
     tkwm.resizable(top, 0, 0)
     tkselection.set(variableBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
+    tkbind(variableBox, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
 
 pieChart <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.factors) == 0){
+        tkmessageBox(message="There are no factors in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Pie Chart")
     variableFrame <- tkframe(top)
@@ -673,7 +824,7 @@ pieChart <- function(){
     for (var in .factors) tkinsert(variableBox, "end", var)
     onOK <- function(){
         variable <- as.character(tkget(variableBox, "active"))
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         command <- (paste("pie(table(", variable, "), labels=levels(",
             variable, '), main="', variable, '", col=rainbow(length(levels(',
@@ -685,13 +836,13 @@ pieChart <- function(){
     buttonsFrame <- tkframe(top)
     OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }
     cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(pie)
         }
     helpButton <- tkbutton(buttonsFrame, text="Help", width="12", command=onHelp)
@@ -708,8 +859,146 @@ pieChart <- function(){
     tkwm.resizable(top, 0, 0)
     tkselection.set(variableBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
+    tkbind(variableBox, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
+    tkfocus(top)
+    tkwait.window(top)
+    }
+
+linePlot <- function(){
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) < 2){
+        tkmessageBox(message="There are fewer than 2 numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
+    top <- tktoplevel()
+    tkwm.title(top, "Line Plot")
+    variablesFrame <- tkframe(top)
+    xFrame <- tkframe(variablesFrame)
+    yFrame <- tkframe(variablesFrame)
+    xBox <- tklistbox(xFrame, height=min(4, length(.numeric)),
+        selectmode="single", background="white", exportselection="FALSE")
+    xScroll <- tkscrollbar(xFrame, repeatinterval=5, command=function(...) tkyview(xBox, ...))
+    tkconfigure(xBox, yscrollcommand=function(...) tkset(xScroll, ...))
+    for (x in .numeric) tkinsert(xBox, "end", x)
+    yBox <- tklistbox(yFrame, height=min(4, length(.numeric)),
+        selectmode="multiple", background="white", exportselection="FALSE")
+    yScroll <- tkscrollbar(yFrame, repeatinterval=5, command=function(...) tkyview(yBox, ...))    
+    tkconfigure(yBox, yscrollcommand=function(...) tkset(yScroll, ...))
+    for (y in .numeric) tkinsert(yBox, "end", y)
+    axisLabelVariable <- tclVar("<use y-variable names>")
+    axisLabelFrame <- tkframe(top)
+    axisLabelEntry <- tkentry(axisLabelFrame, width="40", textvariable=axisLabelVariable)
+    axisLabelScroll <- tkscrollbar(axisLabelFrame, orient="horizontal",
+        repeatinterval=5, command=function(...) tkxview(axisLabelEntry, ...))
+    tkconfigure(axisLabelEntry, xscrollcommand=function(...) tkset(axisLabelScroll, ...))
+    legendFrame <- tkframe(top)
+    legendVariable <- tclVar("0")
+    legendCheckBox <- tkcheckbutton(legendFrame, variable=legendVariable)
+    onOK <- function(){
+        y <- .numeric[as.numeric(tkcurselection(yBox)) + 1]
+        x <- as.character(tkget(xBox, "active"))
+        if (0 == length(y)) {
+            tkmessageBox(message="No y variables selected.", 
+                icon="error", type="ok")
+            if (.grab.focus) tkgrab.release(top)
+            tkdestroy(top)
+            linePlot()
+            return()
+            }
+        if (is.element(x, y)) {
+            tkmessageBox(message="x and y variables must be different.", 
+                icon="error", type="ok")
+            if (.grab.focus) tkgrab.release(top)
+            tkdestroy(top)
+            linePlot()
+            return()
+            }
+        .x <- na.omit(eval(parse(text=x), envir=.GlobalEnv))
+        if (!identical(order(.x), seq(along=.x))){
+            response <- tclvalue(tkmessageBox(message="x-values are not in order.\nContinue?", 
+                icon="warning", type="okcancel", default="cancel"))
+            if (response == "cancel") {
+                onCancel()
+                return()
+                }
+            }
+        axisLabel <- tclvalue(axisLabelVariable)
+        legend <- tclvalue(legendVariable) == "1"
+        if (axisLabel == "<use y-variable names>"){
+            axisLabel <- if (legend) ""
+                else if(length(y) == 1) y
+                else paste(paste("(", 1:length(y), ") ", y, sep=""), collapse=", ")
+            }
+        if (.grab.focus) tkgrab.release(top)
+        tkdestroy(top)
+        pch <- if (length(y) == 1) ", pch=1" else ""
+        command <- paste("matplot(", x, ", ", .activeDataSet, "[, ",
+            paste("c(", paste(paste('"', y, '"', sep=""), collapse=","), ")", sep=""),
+            '], type="b", lty=1, ylab="', axisLabel, '"', pch, ")", sep="")
+        logger(command)
+        justDoIt(command)
+        if (legend && length(y) > 1){
+            n <- length(y)
+            cols <- rep(1:6, 1 + n %/% 6)[1:n]
+            command <- paste("legend(locator(1), legend=", 
+                paste("c(", paste(paste('"', y, '"', sep=""), collapse=","), ")", sep=""),
+                ", col=c(", paste(cols, collapse=","), "), lty=1, pch=c(",
+                paste(paste('"', as.character(1:n), '"', sep=""), collapse=","), "))", sep="")
+            logger(command)
+            justDoIt(command)
+            }
+        tkfocus(.commander)
+        }
+    onCancel <- function() {
+        if (.grab.focus) tkgrab.release(top)
+        tkfocus(.commander)
+        tkdestroy(top)  
+        }
+    buttonsFrame <- tkframe(top)
+    OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
+    cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
+    onHelp <- function() {
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
+        help(matplot)
+        }
+    helpButton <- tkbutton(buttonsFrame, text="Help", width="12", command=onHelp)
+    tkgrid(tklabel(variablesFrame, text="x variable (pick one)"), 
+        tklabel(variablesFrame, text="    "),
+        tklabel(variablesFrame, text="y variables (pick one or more)"), sticky="w")
+    tkgrid(yBox, yScroll, sticky="nw")
+    tkgrid(xBox, xScroll, sticky="nw")
+    tkgrid(xFrame, tklabel(variablesFrame, text="    "), yFrame, sticky="nw")
+    tkgrid(variablesFrame, sticky="w")    
+    tkgrid(OKbutton, cancelButton, tklabel(buttonsFrame, text="            "), 
+        helpButton, sticky="w")
+    tkgrid(tklabel(axisLabelFrame, text="Label for y-axis"), sticky="w")
+    tkgrid(axisLabelEntry, sticky="w")
+    tkgrid(axisLabelScroll, sticky="ew")
+    tkgrid(axisLabelFrame, sticky="w")
+    tkgrid(tklabel(legendFrame, text="Plot legend (position with mouse click)"),
+        legendCheckBox, sticky="w")
+    tkgrid(legendFrame, sticky="w")
+    tkgrid(buttonsFrame, stick="w")
+    tkgrid.configure(helpButton, sticky="e")
+    tkgrid.configure(xScroll, sticky="ns")
+    tkgrid.configure(yScroll, sticky="ns")
+    for (row in 0:3) tkgrid.rowconfigure(top, row, weight=0)
+    for (col in 0:0) tkgrid.columnconfigure(top, col, weight=0)
+    .Tcl("update idletasks")
+    tkwm.resizable(top, 0, 0)
+    tkselection.set(xBox, 0)
+    tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
+    tkwm.deiconify(top)
+#    if (.grab.focus) tkgrab.set(top)  # causes problem ?
     tkfocus(top)
     tkwait.window(top)
     }
