@@ -1,23 +1,38 @@
 # Statistics Menu dialogs
 
-# last modified 11 June 03 by J. Fox
+# last modified 20 July 03 by J. Fox
 
     # Summaries menu
     
 summarizeDataSet <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
     nvar <- length(.variables)
     if (nvar > 10){
         response <- tkmessageBox(message=paste("There are ", nvar, " variables in the data set ",
             .activeDataSet, ".\nDo you want to proceed?", sep=""),
             icon="question", type="okcancel", default="cancel")
-        if ("cancel" == tclvalue(response)) return()
+        if ("cancel" == tclvalue(response)) {
+            tkfocus(.commander)        
+            return()
+            }
         }
     doItAndPrint(paste("summary(", .activeDataSet, ")", sep=""))
     }
 
 numericalSummaries <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) == 0){
+        tkmessageBox(message="There no numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Numerical Summaries")
     xFrame <- tkframe(top)
@@ -41,7 +56,7 @@ numericalSummaries <- function(){
         x <- as.character(tkget(xBox, "active"))
         quants <- paste("c(", gsub(" ", ",", tclvalue(quantiles)), ")")
         var <- paste(.activeDataSet, "$", x, sep="")
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         if (.groups == "FALSE") {
             if (tclvalue(meanVariable) == "1") doItAndPrint(paste("mean(", var, ", na.rm=TRUE)", sep=""))
@@ -72,18 +87,18 @@ numericalSummaries <- function(){
         onOKsub <- function() {
             groups <- as.character(tkget(groupsBox, "active"))
             assign(".groups", groups, envir=.GlobalEnv)
-            tkgrab.release(subdialog)
+            if (.grab.focus) tkgrab.release(subdialog)
             tkdestroy(subdialog)
             tkwm.deiconify(top)
-            tkgrab.set(top)
+            if (.grab.focus) tkgrab.set(top)
             tkfocus(top)
             tkwait.window(top)
             }
         onCancelSub <- function() {
-            tkgrab.release(subdialog)  
+            if (.grab.focus) tkgrab.release(subdialog)  
             tkdestroy(subdialog)
             tkwm.deiconify(top)
-            tkgrab.set(top)
+            if (.grab.focus) tkgrab.set(top)
             tkfocus(top)
             tkwait.window(top)
             }
@@ -102,13 +117,15 @@ numericalSummaries <- function(){
         tkgrid.configure(groupsScroll, sticky="ns")
         tkselection.set(groupsBox, 0)
         tkbind(subdialog, "<Return>", onOKsub)
+        if (.double.click) tkbind(subdialog, "<Double-ButtonPress-1>", onOKsub)
+        tkbind(groupsBox, "<Double-ButtonPress-1>", onOKsub)
         tkwm.deiconify(subdialog)
-        tkgrab.set(subdialog)
+        if (.grab.focus) tkgrab.set(subdialog)
         tkfocus(subdialog)
         tkwait.window(subdialog)
         }
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }
@@ -116,7 +133,7 @@ numericalSummaries <- function(){
     OKbutton <- tkbutton(buttonFrame, text="OK", width="12", command=onOK, default="active")
     cancelButton <- tkbutton(buttonFrame, text="Cancel", width="12",command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(quantile)
         }
     helpButton <- tkbutton(buttonFrame, text="Help", width="12", command=onHelp)
@@ -140,14 +157,24 @@ numericalSummaries <- function(){
     tkwm.resizable(top, 0, 0)
     tkselection.set(xBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
 
 frequencyDistribution <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.factors) == 0){
+        tkmessageBox(message="There no factors in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Frequency Distribution")
     xFrame <- tkframe(top)
@@ -158,7 +185,7 @@ frequencyDistribution <- function(){
     for (x in .factors) tkinsert(xBox, "end", x)
     onOK <- function(){
         x <- as.character(tkget(xBox, "active"))
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         command <- paste("table(", x, ")", sep="")
         logger(paste(".Table <-", command))
@@ -171,7 +198,7 @@ frequencyDistribution <- function(){
         tkfocus(.commander)
         }
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         } 
@@ -179,7 +206,7 @@ frequencyDistribution <- function(){
     OKbutton <- tkbutton(buttonFrame, text="OK", width="12", command=onOK, default="active")
     cancelButton <- tkbutton(buttonFrame, text="Cancel", width="12",command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(table)
         }
     helpButton <- tkbutton(buttonFrame, text="Help", width="12", command=onHelp)
@@ -195,14 +222,31 @@ frequencyDistribution <- function(){
     tkwm.resizable(top, 0, 0)
     tkselection.set(xBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
+    tkbind(xBox, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
 
 statisticsTable <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) == 0){
+        tkmessageBox(message="There no numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.factors) == 0){
+        tkmessageBox(message="There no factors in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Table of Statistics")
     variablesFrame <- tkframe(top)
@@ -233,7 +277,7 @@ statisticsTable <- function(){
         if (0 == length(groups)) {
             tkmessageBox(message="No factors selected.", 
                 icon="error", type="ok")
-            tkgrab.release(top)
+            if (.grab.focus) tkgrab.release(top)
             tkdestroy(top)
             statisticsTable()
             return()
@@ -241,7 +285,7 @@ statisticsTable <- function(){
         response <- as.character(tkget(responseBox, "active"))
         statistic <- tclvalue(statisticVariable)
         if (statistic == "other") statistic <- tclvalue(otherVariable)
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         groups.list <- paste(paste(groups, "=", groups, sep=""), collapse=", ")
         doItAndPrint(paste("tapply(", response, ", list(", groups.list,
@@ -251,13 +295,13 @@ statisticsTable <- function(){
     buttonsFrame <- tkframe(top)
     OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }  
     cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(tapply)
         }
     helpButton <- tkbutton(buttonsFrame, text="Help", width="12", command=onHelp)
@@ -283,14 +327,24 @@ statisticsTable <- function(){
     tkwm.resizable(top, 0, 0)
     tkselection.set(responseBox, 0)
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
     
 correlationMatrix <- function(){
-    if (activeDataSet() == FALSE) return()
+    if (activeDataSet() == FALSE) {
+        tkfocus(.commander)
+        return()
+        }
+    if (length(.numeric) < 2){
+        tkmessageBox(message="There fewer than 2 numeric variables in the active data set.", 
+                icon="error", type="ok")
+        tkfocus(.commander)
+        return()
+        }
     top <- tktoplevel()
     tkwm.title(top, "Correlation Matrix")
     xFrame <- tkframe(top)
@@ -310,7 +364,7 @@ correlationMatrix <- function(){
         if (2 > length(x)) {
             tkmessageBox(message="Fewer than 2 variables selected.", 
                 icon="error", type="ok")
-            tkgrab.release(top)
+            if (.grab.focus) tkgrab.release(top)
             tkdestroy(top)
             correlationMatrix()
             return()
@@ -318,12 +372,12 @@ correlationMatrix <- function(){
         if ((correlations == "partial") && (3 > length(x))) {
             tkmessageBox(message="Fewer than 3 variables selected\nfor partial correlations.", 
                 icon="error", type="ok")
-            tkgrab.release(top)
+            if (.grab.focus) tkgrab.release(top)
             tkdestroy(top)
             correlationMatrix()
             return()
             }
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkdestroy(top)
         x <- paste('"', x, '"', sep="")
         if (correlations == "Pearson")
@@ -339,7 +393,7 @@ correlationMatrix <- function(){
         tkfocus(.commander)
         }
     onCancel <- function() {
-        tkgrab.release(top)
+        if (.grab.focus) tkgrab.release(top)
         tkfocus(.commander)
         tkdestroy(top)  
         }
@@ -347,7 +401,7 @@ correlationMatrix <- function(){
     OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
     cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12",command=onCancel)
     onHelp <- function() {
-        if (.Platform$OS.type != "windows") tkgrab.release(top)
+        if (.Platform$OS.type != "windows") if (.grab.focus) tkgrab.release(top)
         help(cor)
         }
     helpButton <- tkbutton(top, text="Help", width="12", command=onHelp)
@@ -368,8 +422,9 @@ correlationMatrix <- function(){
     tkgrid.configure(helpButton, sticky="e")
     tkgrid.configure(xScroll, sticky="ns")
     tkbind(top, "<Return>", onOK)
+    if (.double.click) tkbind(top, "<Double-ButtonPress-1>", onOK)
     tkwm.deiconify(top)
-    tkgrab.set(top)
+    if (.grab.focus) tkgrab.set(top)
     tkfocus(top)
     tkwait.window(top)
     }
